@@ -10,7 +10,7 @@
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
   different enhancements (see README-NEW).
 
-  modfied 2005,2006 by Bernhard R. Link (see Changelog)
+  modfied 2005,2006,2007 by Bernhard R. Link (see Changelog)
 
 -----------------------------------------------------------------------------*/
 
@@ -86,13 +86,19 @@ typedef struct {
 
 /* structure into which the directory information is read */
 typedef struct {
-	char name[FILENAME_MAX];
+	char *name;
 	const char *magic_type;
 	IconRec icon;
 	Boolean selected;
 	struct stat stats;
 	struct mime_filetype *mime_type;
 	struct magic_modifiers info;
+	/* only set after flInitialize is run: */
+	int width;
+	/* these are only set after a call to FileListReducedFilenames,
+	 * i.e. only in the IconFileList */
+	char *reducedname;
+	size_t reducedlen;
 } FileRec, **FileList;
 
 struct filetype_data {
@@ -107,6 +113,8 @@ typedef enum { Files, Directories, All } FilterType;
 typedef enum { SortByName, SortBySize, SortByMTime } SortType;
 typedef enum { Tree, Icons, Text } DisplayType;
 
+struct ChmodPopupData;
+
 typedef struct _FileWindowRec {
   struct _FileWindowRec *next;
   DisplayType display_type;
@@ -130,6 +138,11 @@ typedef struct _FileWindowRec {
 	  showLength,showDate;
   Widget file_popup_widget, *file_popup_items;
   Widget dir_popup_widget, *dir_popup_items;
+  struct PopupDialog *mkdir, *createFile, *goTo, *rename, *move, *copy, *link,
+		     *select, *filter;
+  struct ChmodPopupData *chmode;
+  struct ConfirmationDialog *confirmationDialog;
+  struct InfoData *info;
 } FileWindowRec, *FileWindowList;
 
 #define P_READ 0x1
@@ -168,7 +181,7 @@ void reSortFileDisplay(FileWindowRec *fw);
 void reDisplayFileWindow(FileWindowRec *fw);
 
 void clearUpdateMarks(void);
-void markForUpdate(String path);
+void markForUpdate(const char *path);
 void intUpdate(void);
 
 void updateStatus(FileWindowRec *fw);
@@ -310,14 +323,12 @@ Pixmap loadFileIcon(const char *name, Cardinal *pm_width, Cardinal *pm_height);
 
 /*--FmChmod------------------------------------------------------------------*/
 
-void createChmodPopup(void);
 FmCallbackProc chmodPopup;
 
 /*--FmConfirm----------------------------------------------------------------*/
 
-extern Boolean aborted;
-void createConfirmPopup(void);
-int confirm(const char *s1, const char *s2, const char *s3);
+int fwConfirm(FileWindowRec *fw, const char *s1, const char *s2, const char *s3, Boolean *aborted);
+int amConfirm(const char *s1, const char *s2, const char *s3, Boolean *aborted);
 
 /*--FmDelete-----------------------------------------------------------------*/
 
@@ -325,10 +336,7 @@ FmCallbackProc deleteItems, emptyDir;
 
 /*--FmErrors-----------------------------------------------------------------*/
 
-void createErrorPopup(void);
-void error(const char *label1, const char *label2);
-void sysError(const char *label);
-void abortXfm(const char *message) __attribute__ ((noreturn));
+#include "FmErrors.h"
 
 /*--FmExec-------------------------------------------------------------------*/
 
@@ -341,7 +349,6 @@ extern int n_exec_maps;
 
 /*--FmInfo-------------------------------------------------------------------*/
 
-void createInfoPopup(void);
 FmCallbackProc infoPopup;
 
 /*--FmMain-------------------------------------------------------------------*/
@@ -420,8 +427,6 @@ void quit(int unused) __attribute__ ((noreturn));
 
 /*---FmPopup-----------------------------------------------------------------*/
 
-void createMainPopups(void);
-
 FmCallbackProc selectPopup, mkdirPopup, createFilePopup, goToPopup,
   renamePopup, movePopup, copyPopup, linkPopup, filterPopup;
 
@@ -473,6 +478,7 @@ struct PopupDialog {
 	Cardinal count;
 	const QuestionRec *questions;
 	void *data;
+	struct PopupDialog **clear_ptr;
 	struct PopupDialogField { char *answer; Widget widget; } fields[];
 };
 
@@ -495,7 +501,8 @@ static inline Widget *createFmMenu(const char *menu_name, const char *menu_label
 }
 Widget *createButtons(const ButtonRec *buttons, Cardinal n_buttons, Widget parent,
 		      XtPointer client_data);
-struct PopupDialog *createPopupQuestions(const char *name, const char *title, Pixmap bitmap, 
+void createPopupQuestions(struct PopupDialog **out, Widget parent,
+		            const char *name, const char *title, Pixmap bitmap,
 			    const QuestionRec *questions, Cardinal n_questions,
 			    const PDButtonRec *buttons, Cardinal n_buttons,
 			    Cardinal dflt_button);
@@ -515,8 +522,8 @@ Widget *createFloatingMenu(const char *menu_name,
 			   Dimension left_margin, Widget parent, 
 			   XtPointer client_data,
 			   Widget *menu_widget);
-char *varPopup(Pixmap icon_bm, char *action);
-void aboutPopup(void);
+char *varPopup(Widget parent, Pixmap icon_bm, char *action);
+void aboutPopup(Widget parent);
 
 void setWMProps(Widget shell);
 
@@ -529,6 +536,7 @@ void initComms(void);
 
 /*--FmOps--------------------------------------------------------------------*/
 
+char *xstrdup(const char *str);
 char *dirConcat(const char *directory,const char *name);
 char *suffixConcat(const char *directory,const char *name);
 char *spaceConcat(const char *str1, const char *str2);
@@ -538,12 +546,14 @@ void fprintexpand(FILE *f, const char *t, const char *c);
 char *strparse(char *s, const char *t, const char *c);
 int fnmatch(String pattern, String name);
 char *fnexpand(char *fn);
+char *dyn_fnexpand(const char *fn);
 char *expanddirectory(const char *dir);
 char *home_and_env_expand(const char *fn);
 int prefix(const char *s, const char *t);
 int exists(const char *path);
 char *searchPath(char *s1, char *p, const char *s2);
 char *searchPathWithSuffix(char *s1, char *p, const char *s2, const char *suffix);
+Boolean cmpCanonical(const char *dir1, const char *dir2);
 
 int create(const char *path, mode_t mode), rcopy(const char *oldpath, const char *newpath),
   rmove(const char *oldpath, const char *newpath), rdel(const char *path);
