@@ -9,7 +9,7 @@
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
   different enhancements (see README-1.4).
 
-  modified 2005,2006 by Bernhard R. Link (see Changelog)
+  modified 2005,2006,2007 by Bernhard R. Link (see Changelog)
 
   Functions & data for handling the chmod feature
  *  ----------------------------------------------------------------------
@@ -38,6 +38,7 @@
 #include <X11/Xaw3d/Label.h>
 #include <X11/Xaw3d/Command.h>
 #include <X11/Xaw3d/Box.h>
+#include <XtHelper.h>
 
 #include "global.h"
 
@@ -63,7 +64,7 @@ typedef struct {
   int value;
 } ChmodItem;
 
-typedef struct {
+typedef struct ChmodPopupData{
   Widget shell;
   Widget label;
   FileWindowRec *fw;
@@ -71,14 +72,13 @@ typedef struct {
   ChmodItem items[3][3];
 } ChmodData;
 
-static ChmodData chmode;
-
 /*---------------------------------------------------------------------------
   Widget Argument lists
 ---------------------------------------------------------------------------*/
 
 static Arg shell_args[] = {
-  { XtNtitle, (XtArgVal) "Change Permissions" }
+  { XtNtitle, (XtArgVal) "Change Permissions" },
+  { XtNtransientFor, (XtArgVal) NULL }
 };
 
 static Arg label_args[] = {
@@ -143,68 +143,68 @@ static const char *small_labels[] = { "r", "w", "x" };
 
 static CallbackProc chmodRestoreCb, chmodOkCb, chmodCancelCb;
 
-static void setupTicks(void)
+static void setupTicks(ChmodData *chmode)
 {
   register int i,j;
   struct stat *stats;
   Pixmap picts=0;
 
-  stats = &chmode.fw->files[chmode.file]->stats;
+  stats = &chmode->fw->files[chmode->file]->stats;
 
-  chmode.items[OWNER][READ].value     = (stats->st_mode) & S_IRUSR;
-  chmode.items[OWNER][WRITE].value    = (stats->st_mode) & S_IWUSR;
-  chmode.items[OWNER][EXECUTE].value  = (stats->st_mode) & S_IXUSR;
+  chmode->items[OWNER][READ].value     = (stats->st_mode) & S_IRUSR;
+  chmode->items[OWNER][WRITE].value    = (stats->st_mode) & S_IWUSR;
+  chmode->items[OWNER][EXECUTE].value  = (stats->st_mode) & S_IXUSR;
 #ifdef ENHANCE_PERMS
-  chmode.items[OWNER][EXECUTE].value  |= (stats->st_mode) & S_ISUID;
+  chmode->items[OWNER][EXECUTE].value  |= (stats->st_mode) & S_ISUID;
 #endif
 
-  chmode.items[GROUP][READ].value     = (stats->st_mode) & S_IRGRP;
-  chmode.items[GROUP][WRITE].value    = (stats->st_mode) & S_IWGRP;
-  chmode.items[GROUP][EXECUTE].value  = (stats->st_mode) & S_IXGRP;
+  chmode->items[GROUP][READ].value     = (stats->st_mode) & S_IRGRP;
+  chmode->items[GROUP][WRITE].value    = (stats->st_mode) & S_IWGRP;
+  chmode->items[GROUP][EXECUTE].value  = (stats->st_mode) & S_IXGRP;
 #ifdef ENHANCE_PERMS
-  chmode.items[GROUP][EXECUTE].value  |= (stats->st_mode) & S_ISGID;
+  chmode->items[GROUP][EXECUTE].value  |= (stats->st_mode) & S_ISGID;
 #endif
 
-  chmode.items[OTHERS][READ].value    = (stats->st_mode) & S_IROTH;
-  chmode.items[OTHERS][WRITE].value   = (stats->st_mode) & S_IWOTH;
-  chmode.items[OTHERS][EXECUTE].value = (stats->st_mode) & S_IXOTH;
+  chmode->items[OTHERS][READ].value    = (stats->st_mode) & S_IROTH;
+  chmode->items[OTHERS][WRITE].value   = (stats->st_mode) & S_IWOTH;
+  chmode->items[OTHERS][EXECUTE].value = (stats->st_mode) & S_IXOTH;
 #ifdef ENHANCE_PERMS
-  chmode.items[OTHERS][EXECUTE].value  |= (stats->st_mode) & S_ISVTX;
+  chmode->items[OTHERS][EXECUTE].value  |= (stats->st_mode) & S_ISVTX;
   for (i=0; i<3; i++) {
     for (j=0; j<2; j++) {
-      XtVaSetValues(chmode.items[i][j].w, XtNbitmap, 
-		    chmode.items[i][j].value ? bm[TICK_BM] : bm[NOTICK_BM],
+      XtVaSetValues(chmode->items[i][j].w, XtNbitmap, 
+		    chmode->items[i][j].value ? bm[TICK_BM] : bm[NOTICK_BM],
 		    NULL);
     }
   }
-  j=(chmode.items[0][2].value & S_IXUSR ? 1:0);
-  j+=(chmode.items[0][2].value & S_ISUID ? 2:0);
+  j=(chmode->items[0][2].value & S_IXUSR ? 1:0);
+  j+=(chmode->items[0][2].value & S_ISUID ? 2:0);
   switch(j) { case 0: picts=bm[NOTICK_BM]; break; case 1: picts=bm[TICK_BM]; break;
 	      case 2: picts=bm[SUID_BM]; break; case 3: picts=bm[sUID_BM]; break;
   }
-  chmode.items[0][2].value=j;
-  XtVaSetValues(chmode.items[0][2].w, XtNbitmap, picts, (char*)0);
+  chmode->items[0][2].value=j;
+  XtVaSetValues(chmode->items[0][2].w, XtNbitmap, picts, (char*)0);
 
-  j=(chmode.items[1][2].value & S_IXGRP ? 1:0);
-  j+=(chmode.items[1][2].value & S_ISGID ? 2:0);
+  j=(chmode->items[1][2].value & S_IXGRP ? 1:0);
+  j+=(chmode->items[1][2].value & S_ISGID ? 2:0);
   switch(j) { case 0: picts=bm[NOTICK_BM]; break; case 1: picts=bm[TICK_BM]; break;
 	      case 2: picts=bm[SUID_BM]; break; case 3: picts=bm[sUID_BM]; break;
   }
-  chmode.items[1][2].value=j;
-  XtVaSetValues(chmode.items[1][2].w, XtNbitmap, picts, (char*)0);
+  chmode->items[1][2].value=j;
+  XtVaSetValues(chmode->items[1][2].w, XtNbitmap, picts, (char*)0);
 
-  j=(chmode.items[2][2].value & S_IXOTH ? 1:0);
-  j+=(chmode.items[2][2].value & S_ISVTX ? 2:0);
+  j=(chmode->items[2][2].value & S_IXOTH ? 1:0);
+  j+=(chmode->items[2][2].value & S_ISVTX ? 2:0);
   switch(j) { case 0: picts=bm[NOTICK_BM]; break; case 1: picts=bm[TICK_BM]; break;
 	      case 2: picts=bm[STICKY_BM]; break; case 3: picts=bm[StICKY_BM]; break;
   }
-  chmode.items[2][2].value=j;
-  XtVaSetValues(chmode.items[2][2].w, XtNbitmap, picts, (char*)0);
+  chmode->items[2][2].value=j;
+  XtVaSetValues(chmode->items[2][2].w, XtNbitmap, picts, (char*)0);
 #else
   for (i=0; i<3; i++) {
     for (j=0; j<3; j++) {
-      XtVaSetValues(chmode.items[i][j].w, XtNbitmap, 
-		    chmode.items[i][j].value ? bm[TICK_BM] : bm[NOTICK_BM],
+      XtVaSetValues(chmode->items[i][j].w, XtNbitmap, 
+		    chmode->items[i][j].value ? bm[TICK_BM] : bm[NOTICK_BM],
 		    NULL);
     }
   }
@@ -213,10 +213,12 @@ static void setupTicks(void)
 
 /*---------------------------------------------------------------------------*/
   
-static void chmodRestoreCb(UNUSED(Widget w), UNUSED(void *client_data),
+static void chmodRestoreCb(UNUSED(Widget w), void *client_data,
 		           UNUSED(XtPointer call_data))
 {
-  setupTicks();
+  ChmodData *chmode = client_data;
+
+  setupTicks(chmode);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -225,29 +227,32 @@ static void tickBoxCb(Widget w, XtPointer client_data,
 		                UNUSED(XtPointer call_data))
 {
   register int i,j;
+  ChmodData *chmode = client_data;
 #ifdef ENHANCE_PERMS
   register int	 k;
   Pixmap picts=0;
 #endif
 
-  i = (int) client_data;
-
-  for (j=0; j<3; j++)
-    if (w == chmode.items[i][j].w)
+  for (i=0; i<3; i++) {
+    for (j=0; j<3; j++)
+      if (w == chmode->items[i][j].w)
+	break;
+    if (j < 3 )
       break;
+  }
 #ifdef ENHANCE_PERMS
   if (j<2) {
 #endif
-  if (chmode.items[i][j].value)
-    chmode.items[i][j].value = False;
+  if (chmode->items[i][j].value)
+    chmode->items[i][j].value = False;
   else
-    chmode.items[i][j].value = True;
+    chmode->items[i][j].value = True;
 
-  XtVaSetValues(chmode.items[i][j].w, XtNbitmap, 
-	       chmode.items[i][j].value ? bm[TICK_BM] : bm[NOTICK_BM], NULL);
+  XtVaSetValues(chmode->items[i][j].w, XtNbitmap, 
+	       chmode->items[i][j].value ? bm[TICK_BM] : bm[NOTICK_BM], NULL);
 #ifdef ENHANCE_PERMS
   } else {
-  k=chmode.items[i][j].value=(chmode.items[i][j].value+1)%4;
+  k=chmode->items[i][j].value=(chmode->items[i][j].value+1)%4;
   if (i<2) 
   switch(k) { case 0: picts=bm[NOTICK_BM]; break; case 1: picts=bm[TICK_BM]; break;
 	      case 2: picts=bm[SUID_BM]; break; case 3: picts=bm[sUID_BM]; break;
@@ -256,20 +261,21 @@ static void tickBoxCb(Widget w, XtPointer client_data,
   switch(k) { case 0: picts=bm[NOTICK_BM]; break; case 1: picts=bm[TICK_BM]; break;
 	      case 2: picts=bm[STICKY_BM]; break; case 3: picts=bm[StICKY_BM]; break;
   }
-  XtVaSetValues(chmode.items[i][j].w, XtNbitmap, picts, (char*)0);
+  XtVaSetValues(chmode->items[i][j].w, XtNbitmap, picts, (char*)0);
   }
 #endif
 }
 
 /*---------------------------------------------------------------------------*/
 
-static void chmodOkCb(UNUSED(Widget w), UNUSED(void *client_data),
+static void chmodOkCb(UNUSED(Widget w), void *client_data,
 		      UNUSED(XtPointer call_data))
 {
   mode_t mode;
+  ChmodData *chmode = client_data;
 
-  XtPopdown(chmode.shell);
-  mode = chmode.fw->files[chmode.file]->stats.st_mode;
+  XtPopdown(chmode->shell);
+  mode = chmode->fw->files[chmode->file]->stats.st_mode;
   mode &= ~(S_IRUSR | S_IWUSR | S_IXUSR |
             S_IRGRP | S_IWGRP | S_IXGRP |
 #ifdef ENHANCE_PERMS
@@ -277,46 +283,43 @@ static void chmodOkCb(UNUSED(Widget w), UNUSED(void *client_data),
 #endif
             S_IROTH | S_IWOTH | S_IXOTH);
 
-  mode |= chmode.items[OWNER][READ].value     ? S_IRUSR : 0;
-  mode |= chmode.items[OWNER][WRITE].value    ? S_IWUSR : 0;
+  mode |= chmode->items[OWNER][READ].value     ? S_IRUSR : 0;
+  mode |= chmode->items[OWNER][WRITE].value    ? S_IWUSR : 0;
 #ifdef ENHANCE_PERMS
-  mode |= chmode.items[OWNER][EXECUTE].value & 1 ? S_IXUSR : 0;
-  mode |= chmode.items[OWNER][EXECUTE].value & 2 ? S_ISUID : 0;
+  mode |= chmode->items[OWNER][EXECUTE].value & 1 ? S_IXUSR : 0;
+  mode |= chmode->items[OWNER][EXECUTE].value & 2 ? S_ISUID : 0;
 #else
-  mode |= chmode.items[OWNER][EXECUTE].value  ? S_IXUSR : 0;
+  mode |= chmode->items[OWNER][EXECUTE].value  ? S_IXUSR : 0;
 #endif
 
-  mode |= chmode.items[GROUP][READ].value     ? S_IRGRP : 0;
-  mode |= chmode.items[GROUP][WRITE].value    ? S_IWGRP : 0;
+  mode |= chmode->items[GROUP][READ].value     ? S_IRGRP : 0;
+  mode |= chmode->items[GROUP][WRITE].value    ? S_IWGRP : 0;
 #ifdef ENHANCE_PERMS
-  mode |= chmode.items[GROUP][EXECUTE].value & 1 ? S_IXGRP : 0;
-  mode |= chmode.items[GROUP][EXECUTE].value & 2 ? S_ISGID : 0;
+  mode |= chmode->items[GROUP][EXECUTE].value & 1 ? S_IXGRP : 0;
+  mode |= chmode->items[GROUP][EXECUTE].value & 2 ? S_ISGID : 0;
 #else
-  mode |= chmode.items[GROUP][EXECUTE].value  ? S_IXGRP : 0;
+  mode |= chmode->items[GROUP][EXECUTE].value  ? S_IXGRP : 0;
 #endif
 
-  mode |= chmode.items[OTHERS][READ].value    ? S_IROTH : 0;
-  mode |= chmode.items[OTHERS][WRITE].value   ? S_IWOTH : 0;
+  mode |= chmode->items[OTHERS][READ].value    ? S_IROTH : 0;
+  mode |= chmode->items[OTHERS][WRITE].value   ? S_IWOTH : 0;
 #ifdef ENHANCE_PERMS
-  mode |= chmode.items[OTHERS][EXECUTE].value & 1 ? S_IXOTH : 0;
-  mode |= chmode.items[OTHERS][EXECUTE].value & 2 ? S_ISVTX : 0;
+  mode |= chmode->items[OTHERS][EXECUTE].value & 1 ? S_IXOTH : 0;
+  mode |= chmode->items[OTHERS][EXECUTE].value & 2 ? S_ISVTX : 0;
 #else
-  mode |= chmode.items[OTHERS][EXECUTE].value ? S_IXOTH : 0;
+  mode |= chmode->items[OTHERS][EXECUTE].value ? S_IXOTH : 0;
 #endif
 
-  if (chdir(chmode.fw->directory)) {
+  if (chdir(chmode->fw->directory)) {
     sysError("System error:");
     goto out;
   }
 
-  if (chmod(chmode.fw->files[chmode.file]->name, mode)) {
-    char s[0xff];
-    sprintf(s, "Can't change modes for %s:", 
-	    chmode.fw->files[chmode.file]->name);
-    sysError(s);
+  if (chmod(chmode->fw->files[chmode->file]->name, mode)) {
+    sysErrorFmt("Can't change modes for %s:", chmode->fw->files[chmode->file]->name);
   }
   else {
-    markForUpdate(chmode.fw->directory);
+    markForUpdate(chmode->fw->directory);
     intUpdate();
   }
 
@@ -326,12 +329,24 @@ static void chmodOkCb(UNUSED(Widget w), UNUSED(void *client_data),
 
 /*---------------------------------------------------------------------------*/
 
-static void chmodCancelCb(UNUSED(Widget w), UNUSED(void *client_data),
+static void chmodCancelCb(UNUSED(Widget w), void *client_data,
 		          UNUSED(XtPointer call_data))
 {
-  XtPopdown(chmode.shell);
+  ChmodData *chmode = client_data;
+
+  XtPopdown(chmode->shell);
   freeze = False;
-}		   
+}
+
+static void chmodDestroy(UNUSED(Widget w), void *client_data,
+		          UNUSED(XtPointer call_data))
+{
+  ChmodData *chmode = client_data;
+
+  if( XtIsBeingDestroyed(XtParent(chmode->shell)))
+      return;
+  chmode->fw->chmode = NULL;
+}
 
 /*---------------------------------------------------------------------------
   Button Information
@@ -347,16 +362,21 @@ static const ButtonRec chmod_buttons[] = {
   PUBLIC FUNCTIONS
 ---------------------------------------------------------------------------*/
 
-void createChmodPopup(void)
+static void createChmodPopup(FileWindowRec *fw)
 {
+  ChmodData *chmode;
   Widget form, form2, blabel, w;
   register int i,j;
 
+  chmode = (void*)XtCalloc(1, sizeof(ChmodData));
+  chmode->fw = fw;
+
   /* create shell */
-  chmode.shell = XtCreatePopupShell("chmod", transientShellWidgetClass,
+  shell_args[1].value = (XtArgVal)fw->shell;
+  chmode->shell = XtCreatePopupShell("chmod", transientShellWidgetClass,
 				   aw.shell, shell_args, XtNumber(shell_args));
   /* create outer form */
-  form = XtCreateManagedWidget("form", formWidgetClass, chmode.shell,
+  form = XtCreateManagedWidget("form", formWidgetClass, chmode->shell,
 			       form_args, XtNumber(form_args) );
 
   /* create two labels for message */
@@ -364,10 +384,10 @@ void createChmodPopup(void)
   label_args[1].value = (XtArgVal) NULL;
   label_args[3].value = (XtArgVal) FORM_WIDTH*3 + 30;
   label_args[4].value = (XtArgVal) resources.label_font;
-  chmode.label = XtCreateManagedWidget("label1", labelWidgetClass, form, 
+  chmode->label = XtCreateManagedWidget("label1", labelWidgetClass, form, 
 				       label_args, XtNumber(label_args) );
 
-  form2_args[1].value = (XtArgVal) chmode.label;
+  form2_args[1].value = (XtArgVal) chmode->label;
   label_args[5].value = (XtArgVal) XtJustifyCenter;
 
   form2 = NULL;
@@ -398,8 +418,8 @@ void createChmodPopup(void)
       tickbox_args[3].value = (XtArgVal) FORM_WIDTH/3;
       w = XtCreateManagedWidget(small_labels[j], commandWidgetClass,
 				form2, tickbox_args, XtNumber(tickbox_args) );
-      XtAddCallback(w, XtNcallback, (XtCallbackProc) tickBoxCb, (XtPointer) i);
-      chmode.items[i][j].w = w;
+      XtAddCallback(w, XtNcallback, (XtCallbackProc) tickBoxCb, chmode);
+      chmode->items[i][j].w = w;
     }
   }
   
@@ -407,15 +427,17 @@ void createChmodPopup(void)
   button_box_args[1].value = (XtArgVal) form2;
   w = XtCreateManagedWidget("button box", boxWidgetClass, form, 
 			    button_box_args, XtNumber(button_box_args) );
-  createButtons(chmod_buttons, XtNumber(chmod_buttons), w, NULL);
+  createButtons(chmod_buttons, XtNumber(chmod_buttons), w, chmode);
 
 #ifdef ENHANCE_POP_ACCEL
   XtInstallAllAccelerators(form,form);
-  XtInstallAllAccelerators(chmode.shell,form);
+  XtInstallAllAccelerators(chmode->shell,form);
 #endif
+  XtAddCallback(chmode->shell, XtNdestroyCallback, chmodDestroy, chmode);
 
-  XtRealizeWidget(chmode.shell);
-  setWMProps(chmode.shell);
+  fw->chmode = chmode;
+  XtRealizeWidget(chmode->shell);
+  setWMProps(chmode->shell);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -430,34 +452,28 @@ void chmodPopup(UNUSED(Widget w), FileWindowRec *fw, UNUSED(void *call_data))
 
   if (!fw->n_selections) return;
 
-  chmode.fw = fw;
+  if( fw->chmode == NULL )
+    createChmodPopup(fw);
 
   for (i=0;; i++)
     if (fw->files[i]->selected) {
-      chmode.file = i;
+      fw->chmode->file = i;
       break;
     }
 
-  l = strlen(chmode.fw->files[chmode.file]->name);
+  l = strlen(fw->files[fw->chmode->file]->name);
 #define msg "Changing access permissions for "
 #define msgsize sizeof(msg)
   message = XtMalloc(msgsize+l);
   memcpy(message, msg, msgsize);
-  memcpy(message+msgsize-1, chmode.fw->files[chmode.file]->name, l+1);
+  memcpy(message+msgsize-1, fw->files[fw->chmode->file]->name, l+1);
 #undef msgsize
 #undef msg
-  XtVaSetValues(chmode.label, XtNlabel, (XtArgVal) message, NULL);
+  XtVaSetValues(fw->chmode->label, XtNlabel, (XtArgVal) message, NULL);
   XtFree(message);
 
-  setupTicks();
+  setupTicks(fw->chmode);
 
   freeze = True;
-  popupByCursor(chmode.shell, XtGrabExclusive);
+  popupByCursor(fw->chmode->shell, XtGrabExclusive);
 }
-
-
-
-
-
-
-
