@@ -10,7 +10,7 @@
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
   different enhancements (see README-1.4).
 
-  modified 2003,2004,2005,2006 by Bernhard R. Link (see Changelog)
+  modified 2003,2004,2005,2006,2007 by Bernhard R. Link (see Changelog)
 
  main module for file manager    
  *  ----------------------------------------------------------------------
@@ -65,6 +65,10 @@
 #include "mime.h"
 #include "mime_start.h"
 
+#ifdef ENHANCE_HISTORY
+#include "FmHistory.h"
+#endif
+
 #ifdef ENHANCE_SELECTION
 #include "FmSelection.h"
 #endif
@@ -107,6 +111,11 @@ XtAppContext app_context;
 /* Update semaphor */
 int freeze = False;
 
+#ifdef ENHANCE_HISTORY
+HistoryList path_history;
+#endif
+
+
 /*---------------------------------------------------------------------------
   Command line options
 ---------------------------------------------------------------------------*/
@@ -511,8 +520,32 @@ int main(int argc, char *argv[])
   XtVaSetValues(aw.shell, XtNiconPixmap, bm_appmgr, NULL);
   XtVaSetValues(aw.shell, XtNiconMask, bm_appmgrmsk, NULL);
 
-  /* create the main popup shells */
-  createMainPopups();
+  /* Create Path History menu, must be before any file window is created */
+#ifdef ENHANCE_HISTORY
+  char *fixed[5];
+  int	nfixed;
+
+  /* Path History menu is initialized
+   * (need to do this first because the history feature
+   * provides an additional action that must be added to the action table
+   * so the different popups can use it :-)
+   */
+
+  nfixed=0;
+  fixed[nfixed++]=user.home;
+#ifdef ENHANCE_USERINFO
+  fixed[nfixed++]=user.cwd;
+#endif
+  fixed[nfixed++]=0;
+  path_history=FmCreateHistoryList(
+	0, /* let the menu's name be "fm_history" */
+	aw.shell,
+	fixed);
+#endif
+
+  /* some additional Popup Windows to initialize (move them into their windows?) */
+  /* Errors */
+  createErrorPopup();
 
 #ifdef ENHANCE_TRANSLATIONS
   if (strcmp(resources.app_defs_version, XFMVERSION)) {
@@ -537,13 +570,15 @@ int main(int argc, char *argv[])
   }
 #else
   /* check for config dir and run xfm.install if appropriate */
-  if (!exists(resources.app_dir))
-    if (confirm("It appears that you are running xfm for the first time.",
+  if (!exists(resources.app_dir)) {
+    Boolean aborted;
+    if (amConfirm("It appears that you are running xfm for the first time.",
 		"Would you like me to run the xfm.install script now",
-		"to create your personal setup?"))
+		"to create your personal setup?", &aborted))
       system("xfm.install -n");
     else if (aborted)
       exit(1);
+  }
 #endif
 
 #ifdef SYSTEMWIDE_DEFAULTS
