@@ -12,7 +12,7 @@
   modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
   different enhancements (see README-1.4).
 
-  modified 2004,2005,2006 by Bernhard R. Link (see Changelog)
+  modified 2004,2005,2006,2007 by Bernhard R. Link (see Changelog)
 
   Routines for creating and managing popup forms and dialog boxes
  *  ----------------------------------------------------------------------
@@ -47,7 +47,6 @@
 #include "FileList.h"
 #include "Fm.h"
 #ifdef ENHANCE_HISTORY
-#include "Am.h" 	/* need to know aw.shell */
 #include "FmHistory.h"
 #endif
 
@@ -55,36 +54,27 @@
 #include "FmSelection.h"
 #endif
 
-/*---------------------------------------------------------------------------
-  STATIC DATA
----------------------------------------------------------------------------*/
-
-typedef struct {
-  struct PopupDialog *mkdir, *createFile, *goTo, *rename, *move, *copy, *link,
-		     *select, *filter;
-} PopupsRec;
-
-static PopupsRec popups;
-
 /*---------------------------------------------------------------------------
   PRIVATE FUNCTIONS
 ---------------------------------------------------------------------------*/
 
-static char *dir_prefix(char *dir, char *path)
+static char *dir_prefix(const char *filename)
 {
-  char *p;
-  if ((p = strrchr(path, '/'))) {
-    strcpy(dir, path);
-    if (p == path)
-      dir[1] = '\0';
-    else
-      dir[p-path] = '\0';
-  } else
-    dir[0] = '\0';
-  return dir;
+  const char *p;
+  char *r;
+
+  p = strrchr(filename, '/');
+  if (p == NULL)
+    p = filename;
+  else if (p == filename)
+      p++;
+  r = strndup(filename, p-filename);
+  if( r == NULL )
+    abortXfm("Out of memory!\n");
+  return r;
 }
 
-static PDCallbackProc 
+static PDCallbackProc
   mkdirOkCb, mkdirCancelCb, createFileOkCb, createFileCancelCb, goToOkCb,
   goToCancelCb, moveOkCb, moveCancelCb, copyOkCb, copyCancelCb, linkOkCb,
   linkCancelCb, selectAddCb, selectRemoveCb, selectCancelCb, selectReplaceCb,
@@ -93,19 +83,17 @@ static PDCallbackProc
 static void mkdirOkCb(UNUSED(Widget w), struct PopupDialog *p,
                       UNUSED(void *call_data))
 {
-  char *dir = getPopupAnswer(p,0);
+  char *dir = dyn_fnexpand(getPopupAnswer(p,0));
   FileWindowRec *fw = p->data;
 
   popdownPopup(p);
-  fnexpand(dir);
   if (chdir(fw->directory))
     sysError("System error:");
   else if (mkdir(dir, user.umask & 0777)) {
-    char s[0xff];
-    sprintf(s, "Error creating folder %s:", dir);
-    sysError(s);
+    sysErrorFmt("Error creating folder %s:", dir);
   } else
     intUpdate();
+  XtFree(dir);
   freeze = False;
 }
 
@@ -123,19 +111,17 @@ static void mkdirCancelCb(UNUSED(Widget w), struct PopupDialog *p,
 static void createFileOkCb(UNUSED(Widget w), struct PopupDialog *p,
 		           UNUSED(void *call_data))
 {
-  char *fn = getPopupAnswer(p,0);
+  char *fn = dyn_fnexpand(getPopupAnswer(p,0));
   FileWindowRec *fw = p->data;
 
   popdownPopup(p);
-  fnexpand(fn);
   if (chdir(fw->directory))
     sysError("System error:");
   else if (create(fn, user.umask & 0666)) {
-    char s[0xff];
-    sprintf(s, "Error creating file %s:", fn);
-    sysError(s);
+    sysErrorFmt("Error creating file %s:", fn);
   } else
     intUpdate();
+  XtFree(fn);
   freeze = False;
 }
 
@@ -153,7 +139,7 @@ static void createFileCancelCb(UNUSED(Widget w), struct PopupDialog *p,
 static void goToOkCb(UNUSED(Widget w), struct PopupDialog *p,
 		     UNUSED(void *call_data))
 {
-  char *dir = getPopupAnswer(p,0);
+  char *dir = dyn_fnexpand(getPopupAnswer(p,0));
   FileWindowRec *fw = p->data;
   char path[MAXPATHLEN];
 #ifdef ENHANCE_SCROLL
@@ -161,7 +147,6 @@ static void goToOkCb(UNUSED(Widget w), struct PopupDialog *p,
 #endif
 
   popdownPopup(p);
-  fnexpand(dir);
 #ifdef ENHANCE_BUGFIX
   /* if it's impossible to chdir(popups.fw->directory)
    * it still might be possible to chdir(popups.goTo_s) !
@@ -170,18 +155,13 @@ static void goToOkCb(UNUSED(Widget w), struct PopupDialog *p,
    */
   // TODO: this breaks chdir to relative directories, doesn't it?
   if (chdir(dir)) {
-    char s[0xff];
-    sprintf(s, "Can't open folder %s:", dir);
-    sysError(s);
+    sysErrorFmt("Can't open folder %s:", dir);
 	if (chdir(fw->directory)) {
-    	  sprintf(s, "Can't open folder %s:", fw->directory);
-    	  sysError(s);
+    	  sysErrorFmt("Can't open folder %s:", fw->directory);
 	}
 #else
   if (chdir(popups.fw->directory) || chdir(dir)) {
-    char s[0xff];
-    sprintf(s, "Can't open folder %s:", dir);
-    sysError(s);
+    sysErrorFmt("Can't open folder %s:", dir);
 #endif
   } else if (!getwd(path))
     sysError("System error:");
@@ -196,6 +176,7 @@ static void goToOkCb(UNUSED(Widget w), struct PopupDialog *p,
     updateFileDisplay(fw);
 #endif
   }
+  XtFree(dir);
   freeze = False;
 }
 
@@ -216,11 +197,10 @@ static void renameOkCb(UNUSED(Widget w), struct PopupDialog *p,
   FileWindowRec *fw = p->data;
   struct stat stats;
   int i;
-  char *from = NULL, to[MAXPATHLEN], todir[MAXPATHLEN];
+  char *from = NULL, *to;
 
   popdownPopup(p);
-  strcpy(to, getPopupAnswer(p,0));
-  fnexpand(to);
+  to = dyn_fnexpand(getPopupAnswer(p,0));
 
   if (chdir(fw->directory))
 
@@ -235,6 +215,8 @@ static void renameOkCb(UNUSED(Widget w), struct PopupDialog *p,
 	from = fw->files[i]->name;
 	break;
       }
+    if (from == NULL )
+      goto out;
 
     if (!strcmp(from, ".") || !strcmp(from, "..")) {
       error("Cannot rename . or ..", "");
@@ -246,27 +228,23 @@ static void renameOkCb(UNUSED(Widget w), struct PopupDialog *p,
     }
 
     if (exists(to) && resources.confirm_overwrite) {
-      char s[0xff];
-      sprintf(s, "Rename: file %s already exists", to);
-      if (!confirm(s, "Overwrite?", ""))
+      if (!fwConfirm(fw, "Rename: file already exists!", to, "Overwrite?", NULL))
 	goto out;
     }
 
     if (rmove(from, to)) {
-      char s[0xff];
-      sprintf(s, "Error renaming %s:", from);
-      sysError(s);
+      sysErrorFmt("Error renaming %s:", from);
     } else {
-      dir_prefix(todir, to);
-      if ((*todir?chdir(todir):0) || !getwd(todir))
-	sysError("System error:");
+      char *todir = dir_prefix(to);
       markForUpdate(fw->directory); markForUpdate(todir);
+      XTFREE(to);
       intUpdate();
     }
 
   }
 
  out:
+  XTFREE(to);
   freeze = False;
 }
 
@@ -287,12 +265,13 @@ static void moveOkCb(UNUSED(Widget w), struct PopupDialog *p,
   char *move = getPopupAnswer(p,0);
   FileWindowRec *fw = p->data;
   struct stat stats;
-  int i, toi, n_moved = 0;
-  char *from = NULL, to[MAXPATHLEN], todir[MAXPATHLEN];
+  int i,  n_moved = 0;
+  const char *from = NULL;
+  char *destination;
+  Boolean aborted;
 
   popdownPopup(p);
-  strcpy(to, move);
-  fnexpand(to);
+  destination = dyn_fnexpand(move);
 
   if (chdir(fw->directory))
 
@@ -303,24 +282,14 @@ static void moveOkCb(UNUSED(Widget w), struct PopupDialog *p,
     /* if target exists and is a directory, move the source into that
        directory */
 
-    if (!stat(to, &stats) && S_ISDIR(stats.st_mode)) {
+    if (!stat(destination, &stats) && S_ISDIR(stats.st_mode)) {
+      char *to;
 
-      if (chdir(to) || !getwd(to) || chdir(fw->directory)) {
-	sysError("System error:");
-	goto out;
-      } else if (!strcmp(fw->directory, to)) {
+      if (cmpCanonical(fw->directory, destination)) {
 	error("Move:", "Source and destination are identical");
 	goto out;
       }
 
-      strcpy(todir, to);
-
-      toi = strlen(to);
-      if (to[toi-1] != '/') {
-	to[toi++] = '/';
-	to[toi] = '\0';
-      }
-
       for (i=0; i < fw->n_files; i++)
 	if (fw->files[i]->selected) {
 	  if (!strcmp(fw->files[i]->name, ".") ||
@@ -329,24 +298,26 @@ static void moveOkCb(UNUSED(Widget w), struct PopupDialog *p,
 	    continue;
 	  }
 	  from = fw->files[i]->name;
-	  strcpy(to+toi, from);
+	  to = dirConcat(destination, fw->files[i]->name);
 	  if (exists(to) && resources.confirm_overwrite) {
-	    char s[0xff];
-	    sprintf(s, "Move: file %s already exists at destination", from);
-	    if (!confirm(s, "Overwrite?", "")) {
+	    if (!fwConfirm(fw, "Move: file already exists at destination", from, "Overwrite?", &aborted)) {
+	      XTFREE(to);
 	      if (aborted)
 		break;
 	      else
 		continue;
 	    }
 	  }
-	  if (rmove(from,to)) {
-	    char s[0xff];
-	    sprintf(s, "Error moving %s:", from);
-	    sysError(s);
+	  if (rmove(from, to)) {
+	    sysErrorFmt("Error moving %s:", from);
 	  } else
 	    n_moved++;
+	  XTFREE(to);
 	}
+      if (n_moved) {
+	markForUpdate(fw->directory); markForUpdate(destination);
+	intUpdate();
+      }
     }
 
     /* otherwise only a single file may be selected; move it to the target
@@ -367,42 +338,36 @@ static void moveOkCb(UNUSED(Widget w), struct PopupDialog *p,
 	  break;
 	}
 
+      assert(from != NULL);
+
       if (!strcmp(from, ".") || !strcmp(from, "..")) {
 	error("Cannot move . or ..", "");
 	goto out;
-      } else if (!lstat(to, &stats) && !lstat(from, &stats1) &&
+      } else if (!lstat(destination, &stats) && !lstat(from, &stats1) &&
 		 stats.st_ino == stats1.st_ino) {
 	error("Move:", "Source and destination are identical");
 	goto out;
       }
 
-      if (exists(to) && resources.confirm_overwrite) {
-	char s[0xff];
-	sprintf(s, "Move: file %s already exists", to);
-	if (!confirm(s, "Overwrite?", ""))
+      if (exists(destination) && resources.confirm_overwrite) {
+	if (!fwConfirm(fw, "Move: file already exists", destination, "Overwrite?", NULL))
 	  goto out;
       }
 
-      if (rmove(from, to)) {
-	char s[0xff];
-	sprintf(s, "Error moving %s:", from);
-	sysError(s);
+      if (rmove(from, destination)) {
+	sysErrorFmt("Error moving %s:", from);
       } else {
-	n_moved = 1;
-	dir_prefix(todir, to);
-	if ((*todir?chdir(todir):0) || !getwd(todir))
-	  sysError("System error:");
+	char *todir = dir_prefix(destination);
+	markForUpdate(todir);
+	XTFREE(todir);
+	markForUpdate(fw->directory);
+	intUpdate();
       }
     }
-
-    if (n_moved) {
-      markForUpdate(fw->directory); markForUpdate(todir);
-      intUpdate();
-    }
-
   }
 
  out:
+  XTFREE(destination);
   freeze = False;
 }
 
@@ -423,12 +388,13 @@ static void copyOkCb(UNUSED(Widget w), struct PopupDialog *p,
   char *copy = getPopupAnswer(p,0);
   FileWindowRec *fw = p->data;
   struct stat stats;
-  int i, toi, n_copied = 0;
-  char *from = NULL, to[MAXPATHLEN], todir[MAXPATHLEN];
+  int i, n_copied = 0;
+  const char *from = NULL;
+  char *destination;
+  Boolean aborted;
 
   popdownPopup(p);
-  strcpy(to, copy);
-  fnexpand(to);
+  destination = dyn_fnexpand(copy);
 
   if (chdir(fw->directory))
 
@@ -439,24 +405,14 @@ static void copyOkCb(UNUSED(Widget w), struct PopupDialog *p,
     /* if target exists and is a directory, copy the source into that
        directory */
 
-    if (!stat(to, &stats) && S_ISDIR(stats.st_mode)) {
+    if (!stat(destination, &stats) && S_ISDIR(stats.st_mode)) {
+      char *to;
 
-      if (chdir(to) || !getwd(to) || chdir(fw->directory)) {
-	sysError("System error:");
-	goto out;
-      } else if (!strcmp(fw->directory, to)) {
+      if (cmpCanonical(fw->directory, destination)) {
 	error("Copy:", "Source and destination are identical");
 	goto out;
       }
 
-      strcpy(todir, to);
-
-      toi = strlen(to);
-      if (to[toi-1] != '/') {
-	to[toi++] = '/';
-	to[toi] = '\0';
-      }
-
       for (i=0; i < fw->n_files; i++)
 	if (fw->files[i]->selected) {
 	  if (!strcmp(fw->files[i]->name, ".") ||
@@ -465,11 +421,10 @@ static void copyOkCb(UNUSED(Widget w), struct PopupDialog *p,
 	    continue;
 	  }
 	  from = fw->files[i]->name;
-	  strcpy(to+toi, from);
+	  to = dirConcat(destination, from);
 	  if (exists(to) && resources.confirm_overwrite) {
-	    char s[0xff];
-	    sprintf(s, "Copy: file %s already exists at destination", from);
-	    if (!confirm(s, "Overwrite?", "")) {
+	    if (!fwConfirm(fw, "Copy: file already exists at destination", from, "Overwrite?", &aborted)) {
+	      XTFREE(to);
 	      if (aborted)
 		break;
 	      else
@@ -477,12 +432,15 @@ static void copyOkCb(UNUSED(Widget w), struct PopupDialog *p,
 	    }
 	  }
 	  if (rcopy(from,to)) {
-	    char s[0xff];
-	    sprintf(s, "Error copying %s:", from);
-	    sysError(s);
+	    sysErrorFmt("Error copying %s:", from);
 	  } else
 	    n_copied++;
+	  XTFREE(to);
 	}
+      if (n_copied) {
+	markForUpdate(destination);
+	intUpdate();
+      }
     }
 
     /* otherwise only a single file may be selected; copy it to the target
@@ -506,39 +464,31 @@ static void copyOkCb(UNUSED(Widget w), struct PopupDialog *p,
       if (!strcmp(from, ".") || !strcmp(from, "..")) {
 	error("Cannot copy . or ..", "");
 	goto out;
-      } else if (!lstat(to, &stats) && !lstat(from, &stats1) &&
+      } else if (!lstat(destination, &stats) && !lstat(from, &stats1) &&
 		 stats.st_ino == stats1.st_ino) {
 	error("Copy:", "Source and destination are identical");
 	goto out;
       }
 
-      if (exists(to) && resources.confirm_overwrite) {
-	char s[0xff];
-	sprintf(s, "Copy: file %s already exists", to);
-	if (!confirm(s, "Overwrite?", ""))
+      if (exists(destination) && resources.confirm_overwrite) {
+	if (!fwConfirm(fw, "Copy: file already exists", destination, "Overwrite?", NULL))
 	  goto out;
       }
 
-      if (rcopy(from, to)) {
-	char s[0xff];
-	sprintf(s, "Error copying %s:", from);
-	sysError(s);
+      if (rcopy(from, destination)) {
+	sysErrorFmt("Error copying %s:", from);
       } else {
-	n_copied = 1;
-	dir_prefix(todir, to);
-	if ((*todir?chdir(todir):0) || !getwd(todir))
-	  sysError("System error:");
+	char *todir = dir_prefix(destination);
+	markForUpdate(todir);
+	XTFREE(todir);
+	intUpdate();
       }
     }
 
-    if (n_copied) {
-      markForUpdate(todir);
-      intUpdate();
-    }
-
   }
 
  out:
+  XTFREE(destination);
   freeze = False;
 }
 
@@ -558,20 +508,12 @@ static void linkOkCb(UNUSED(Widget w), struct PopupDialog *p,
 {
   FileWindowRec *fw = p->data;
   struct stat stats;
-  int i, namei, toi, n_linked = 0;
-  char *from = NULL, name[MAXPATHLEN], to[MAXPATHLEN], todir[MAXPATHLEN];
+  int i, n_linked = 0;
+  char *destination;
+  Boolean aborted;
 
   popdownPopup(p);
-  strcpy(to, getPopupAnswer(p,0));
-  fnexpand(to);
-
-  strcpy(name, fw->directory);
-
-  namei = strlen(name);
-  if (name[namei-1] != '/') {
-    name[namei++] = '/';
-    name[namei] = '\0';
-  }
+  destination = dyn_fnexpand(getPopupAnswer(p,0));
 
   if (chdir(fw->directory))
 
@@ -582,33 +524,22 @@ static void linkOkCb(UNUSED(Widget w), struct PopupDialog *p,
     /* if target exists and is a directory, link the source into that
        directory */
 
-    if (!stat(to, &stats) && S_ISDIR(stats.st_mode)) {
+    if (!stat(destination, &stats) && S_ISDIR(stats.st_mode)) {
+      char *to;
 
-      if (chdir(to) || !getwd(to) || chdir(fw->directory)) {
-	sysError("System error:");
-	goto out;
-      } else if (!strcmp(fw->directory, to)) {
+      if (cmpCanonical(fw->directory, destination)) {
 	error("Link:", "Source and destination are identical");
 	goto out;
       }
 
-      strcpy(todir, to);
-
-      toi = strlen(to);
-      if (to[toi-1] != '/') {
-	to[toi++] = '/';
-	to[toi] = '\0';
-      }
-
       for (i=0; i < fw->n_files; i++)
 	if (fw->files[i]->selected) {
-	  from = fw->files[i]->name;
-	  strcpy(name+namei, from);
-	  strcpy(to+toi, from);
+	  const char *from = fw->files[i]->name;
+	  char *name = dirConcat(fw->directory, from);
+	  to = dirConcat(destination, from);
 	  if (exists(to) && resources.confirm_overwrite) {
-	    char s[0xff];
-	    sprintf(s, "Link: file %s already exists at destination", from);
-	    if (!confirm(s, "Overwrite?", "")) {
+	    if (!fwConfirm(fw, "Link: file already exists at destination", from, "Overwrite?", &aborted)) {
+	      XTFREE(to);
 	      if (aborted)
 		break;
 	      else
@@ -616,12 +547,15 @@ static void linkOkCb(UNUSED(Widget w), struct PopupDialog *p,
 	    }
 	  }
 	  if (symlink(name,to)) {
-	    char s[0xff];
-	    sprintf(s, "Error linking %s:", from);
-	    sysError(s);
+	    sysErrorFmt("Error linking %s:", from);
 	  } else
 	    n_linked++;
+	  XTFREE(to);
 	}
+      if (n_linked) {
+	markForUpdate(destination);
+	intUpdate();
+      }
     }
 
     /* otherwise only a single file may be selected; link it to the target
@@ -635,6 +569,8 @@ static void linkOkCb(UNUSED(Widget w), struct PopupDialog *p,
     } else {
 
       struct stat stats1;
+      const char *from = NULL;
+      char *name;
 
       for (i = 0; i < fw->n_files; i++)
 	if (fw->files[i]->selected) {
@@ -642,41 +578,38 @@ static void linkOkCb(UNUSED(Widget w), struct PopupDialog *p,
 	  break;
 	}
 
-      strcpy(name+namei, from);
+      assert(from != NULL);
 
-      if (!lstat(to, &stats) && !lstat(from, &stats1) &&
+      name = dirConcat(fw->directory, from);
+
+      if (!lstat(destination, &stats) && !lstat(from, &stats1) &&
 		 stats.st_ino == stats1.st_ino) {
 	error("Link:", "Source and destination are identical");
+	XTFREE(name);
 	goto out;
       }
 
-      if (exists(to) && resources.confirm_overwrite) {
-	char s[0xff];
-	sprintf(s, "Link: file %s already exists", to);
-	if (!confirm(s, "Overwrite?", ""))
+      if (exists(destination) && resources.confirm_overwrite) {
+	if (!fwConfirm(fw, "Link: file already exists", destination, "Overwrite?", NULL)) {
+	  XTFREE(name);
 	  goto out;
+	}
       }
 
-      if (symlink(name, to)) {
-	char s[0xff];
-	sprintf(s, "Error linking %s:", from);
-	sysError(s);
+      if (symlink(name, destination)) {
+	sysErrorFmt("Error linking %s:", from);
       } else {
-	n_linked = 1;
-	dir_prefix(todir, to);
-	if ((*todir?chdir(todir):0) || !getwd(todir))
-	  sysError("System error:");
+	char *todir = dir_prefix(destination);
+	markForUpdate(todir);
+	XTFREE(destination);
+	intUpdate();
       }
+      XTFREE(name);
     }
-
-    if (n_linked) {
-      markForUpdate(todir);
-      intUpdate();
-    }
-
   }
 
  out:
+  XTFREE(destination);
   freeze = False;
 }
 
@@ -957,124 +890,23 @@ static const QuestionRec filter_questions[] = {             /* KMR */
   { "Filename pattern:", MAXPATHLEN }
 };
 
-#ifdef ENHANCE_HISTORY
-/*--------------------------------------------------------------------------
-  PUBLIC VARIABLES
---------------------------------------------------------------------------*/
-
-HistoryList path_history;
-
-#endif
-
 /*---------------------------------------------------------------------------
   PUBLIC FUNCTIONS
 ---------------------------------------------------------------------------*/
 
-void createMainPopups()
-{
-#ifdef ENHANCE_HISTORY
-  char *fixed[5];
-  int	nfixed;
-
-  /* Path History menu is initialized
-   * (need to do this first because the history feature
-   * provides an additional action that must be added to the action table
-   * so the different popups can use it :-)
-   */
-
-  nfixed=0;
-  fixed[nfixed++]=user.home;
-#ifdef ENHANCE_USERINFO
-  fixed[nfixed++]=user.cwd;
-#endif
-  fixed[nfixed++]=0;
-  path_history=FmCreateHistoryList(
-	0, /* let the menu's name be "fm_history" */
-	aw.shell,
-	fixed);
-
-  /* Note: important resources of the history menu are _not_ configured
-   *       statically but by the resource files.
-   */
-#endif
-
-  /* New Folder */
-  popups.mkdir = createPopupQuestions("mkdir", "New Folder", bm[DIR_BM], 
-			       mkdir_questions, XtNumber(mkdir_questions),
-			       mkdir_buttons, XtNumber(mkdir_buttons),
-			       XtNumber(mkdir_buttons)-1);
-
-  /* New File */
-  popups.createFile = createPopupQuestions("create", "New File", bm[FILE_BM],
-			       createFile_questions,
-			       XtNumber(createFile_questions),
-			       createFile_buttons,
-			       XtNumber(createFile_buttons),
-			       XtNumber(createFile_buttons)-1);
-
-  /* Change current folder */
-  popups.goTo = createPopupQuestions("goto", "Go To", bm[DIR_BM],
-			       goTo_questions, XtNumber(goTo_questions),
-			       goTo_buttons, XtNumber(goTo_buttons),
-			       XtNumber(goTo_buttons)-1);
-
-  /* Rename file */
-  popups.rename = createPopupQuestions("rename", "Rename", bm[FILE_BM], 
-			       rename_questions, XtNumber(rename_questions),
-			       rename_buttons, XtNumber(rename_buttons),
-			       XtNumber(rename_buttons)-1);
-
-  /* Move file */
-  popups.move = createPopupQuestions("move", "Move", bm[FILES_BM], 
-			       move_questions, XtNumber(move_questions),
-			       move_buttons, XtNumber(move_buttons),
-			       XtNumber(move_buttons)-1);
-
-  /* Copy file */
-  popups.copy = createPopupQuestions("copy", "Copy", bm[FILES_BM],
-			       copy_questions, XtNumber(copy_questions),
-			       copy_buttons, XtNumber(copy_buttons),
-			       XtNumber(copy_buttons)-1);
-
-  /* Create link */
-  popups.link = createPopupQuestions("link", "Link", bm[SYMLNK_BM],
-			       link_questions, XtNumber(link_questions),
-			       link_buttons, XtNumber(link_buttons),
-			       XtNumber(link_buttons)-1);
-
-  /* Select */
-  popups.select = createPopupQuestions("select", "Select", bm[FILES_BM],
-			       select_questions, XtNumber(select_questions),
-			       select_buttons, XtNumber(select_buttons),
-			       XtNumber(select_buttons)-1);
-
-  /* Filter */  /* KMR */
-  popups.filter = createPopupQuestions("filter", "Filter", bm[FILES_BM],
-			       filter_questions, XtNumber(filter_questions),
-			       filter_buttons, XtNumber(filter_buttons),
-			       XtNumber(filter_buttons)-1);
-
-  /* Change Access Mode */
-  createChmodPopup();
-
-  /* Info */
-  createInfoPopup();
-
-  /* Errors */
-  createErrorPopup();
-
-  /* Deletions */
-  createConfirmPopup();
-
-}
-
 /*---------------------------------------------------------------------------*/ 
 
 void selectPopup(UNUSED(Widget w), FileWindowRec *fw, UNUSED(void *call_data))
 {
-  reloadPopupData(popups.select);
+  if (fw->select == NULL)
+  	createPopupQuestions(&fw->select, fw->shell,
+			"select", "Select", bm[FILES_BM],
+			select_questions, XtNumber(select_questions),
+			select_buttons, XtNumber(select_buttons),
+			XtNumber(select_buttons)-1);
+  reloadPopupData(fw->select);
   freeze = True;
-  popupPopupByCursor(popups.select, fw);
+  popupPopupByCursor(fw->select, fw);
 }
 
 /*---------------------------------------------------------------------------*/ 
@@ -1082,36 +914,63 @@ void selectPopup(UNUSED(Widget w), FileWindowRec *fw, UNUSED(void *call_data))
 /* KMR */
 void filterPopup(UNUSED(Widget w), FileWindowRec *fw, UNUSED(void *call_data))
 {
-  setPopupData(popups.filter,0,fw->dirFilter);
+  /* Filter */  /* KMR */
+  if (fw->filter == NULL)
+	  createPopupQuestions(&fw->filter, fw->shell,
+			  "filter", "Filter", bm[FILES_BM],
+			  filter_questions, XtNumber(filter_questions),
+			  filter_buttons, XtNumber(filter_buttons),
+			  XtNumber(filter_buttons)-1);
+  setPopupData(fw->filter, 0, fw->dirFilter);
   freeze = True;
-  popupPopupByCursor(popups.filter, fw);
+  popupPopupByCursor(fw->filter, fw);
 }
 
 /*---------------------------------------------------------------------------*/
 
 void mkdirPopup(UNUSED(Widget w), FileWindowRec *fw, UNUSED(void *call_data))
 {
-  reloadPopupData(popups.mkdir);
+  if (fw->mkdir == NULL)
+	  createPopupQuestions(&fw->mkdir, fw->shell,
+			  "mkdir", "New Folder", bm[DIR_BM], 
+			  mkdir_questions, XtNumber(mkdir_questions),
+			  mkdir_buttons, XtNumber(mkdir_buttons),
+			  XtNumber(mkdir_buttons)-1);
+  reloadPopupData(fw->mkdir);
   freeze = True;
-  popupPopupByCursor(popups.mkdir, fw);
+  popupPopupByCursor(fw->mkdir, fw);
 }
 
 /*---------------------------------------------------------------------------*/
 
 void createFilePopup(UNUSED(Widget w), FileWindowRec *fw, UNUSED(void *call_data))
 {
-  reloadPopupData(popups.createFile);
+  if (fw->createFile == NULL)
+	  createPopupQuestions(&fw->createFile, fw->shell,
+			  "create", "New File", bm[FILE_BM],
+			  createFile_questions,
+			  XtNumber(createFile_questions),
+			  createFile_buttons,
+			  XtNumber(createFile_buttons),
+			  XtNumber(createFile_buttons)-1);
+  reloadPopupData(fw->createFile);
   freeze = True;
-  popupPopupByCursor(popups.createFile, fw);
+  popupPopupByCursor(fw->createFile, fw);
 }
 
 /*---------------------------------------------------------------------------*/
 
 void goToPopup(UNUSED(Widget w), FileWindowRec *fw, UNUSED(void *call_data))
 {
-  reloadPopupData(popups.goTo);
+  if (fw->goTo == NULL)
+	  createPopupQuestions(&fw->goTo, fw->shell,
+			  "goto", "Go To", bm[DIR_BM],
+			  goTo_questions, XtNumber(goTo_questions),
+			  goTo_buttons, XtNumber(goTo_buttons),
+			  XtNumber(goTo_buttons)-1);
+  reloadPopupData(fw->goTo);
   freeze = True;
-  popupPopupByCursor(popups.goTo, fw);
+  popupPopupByCursor(fw->goTo, fw);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1124,13 +983,20 @@ void renamePopup(UNUSED(Widget w), FileWindowRec *fw, UNUSED(void *call_data))
 
   if (!fw->n_selections) return;
 
+  if (fw->rename == NULL)
+	  createPopupQuestions(&fw->rename, fw->shell,
+			  "rename", "Rename", bm[FILE_BM], 
+			  rename_questions, XtNumber(rename_questions),
+			  rename_buttons, XtNumber(rename_buttons),
+			  XtNumber(rename_buttons)-1);
+
   for (i = 0; i < fw->n_files; i++)
     if (fw->files[i]->selected) {
-      setPopupData(popups.rename,0,fw->files[i]->name);
+      setPopupData(fw->rename, 0, fw->files[i]->name);
       break;
     }
   freeze = True;
-  popupPopupByCursor(popups.rename, fw);
+  popupPopupByCursor(fw->rename, fw);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1141,9 +1007,16 @@ void movePopup(UNUSED(Widget w), FileWindowRec *fw, UNUSED(void *call_data))
 
   if (!fw->n_selections) return;
 
-  reloadPopupData(popups.move);
+  if (fw->move == NULL)
+	  createPopupQuestions(&fw->move, fw->shell,
+			  "move", "Move", bm[FILES_BM], 
+			  move_questions, XtNumber(move_questions),
+			  move_buttons, XtNumber(move_buttons),
+			  XtNumber(move_buttons)-1);
+
+  reloadPopupData(fw->move);
   freeze = True;
-  popupPopupByCursor(popups.move, fw);
+  popupPopupByCursor(fw->move, fw);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1154,9 +1027,15 @@ void copyPopup(UNUSED(Widget w), FileWindowRec *fw, UNUSED(void *call_data))
 
   if (!fw->n_selections) return;
 
-  reloadPopupData(popups.copy);
+  if (fw->copy == NULL)
+	  createPopupQuestions(&fw->copy, fw->shell,
+			  "copy", "Copy", bm[FILES_BM],
+			  copy_questions, XtNumber(copy_questions),
+			  copy_buttons, XtNumber(copy_buttons),
+			  XtNumber(copy_buttons)-1);
+  reloadPopupData(fw->copy);
   freeze = True;
-  popupPopupByCursor(popups.copy, fw);
+  popupPopupByCursor(fw->copy, fw);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1167,7 +1046,14 @@ void linkPopup(UNUSED(Widget w), FileWindowRec *fw, UNUSED(void *call_data))
 
   if (!fw->n_selections) return;
 
-  reloadPopupData(popups.link);
+  if (fw->link == NULL)
+	  createPopupQuestions(&fw->link, fw->shell,
+			  "link", "Link", bm[SYMLNK_BM],
+			  link_questions, XtNumber(link_questions),
+			  link_buttons, XtNumber(link_buttons),
+			  XtNumber(link_buttons)-1);
+
+  reloadPopupData(fw->link);
   freeze = True;
-  popupPopupByCursor(popups.link, fw);
+  popupPopupByCursor(fw->link, fw);
 }
