@@ -5,7 +5,7 @@
  
  *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>
  *
- *  modified 2005,2006 by Bernhard R. Link (see Changelog)
+ *  modified 2005,2006,2007 by Bernhard R. Link (see Changelog)
  *  ----------------------------------------------------------------------
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -210,6 +210,8 @@ static void ilInitialize(UNUSED(Widget treq), Widget tnew,
  bm_wmax+=2*FL.border_width;
  IL.entry_width = IL.entry_sep + max(bm_wmax,
 			min(FL.name_w, IL.max_label_width));
+ /* Must be called as it initialized width, reducedname and reducedlen */
+ FileListReduceFilenames((FileListWidget)ilw, IL.entry_width - IL.entry_sep);
 
  IL.liney=bm_hmax+IL.labelsep+FL.font->max_bounds.ascent+FL.border_width;
 
@@ -290,7 +292,7 @@ static void ilExpose(Widget w, XEvent *ev, UNUSED(Region reg))
 static void drawPart(IconFileListWidget ilw, 
 	int xmin, int xmax, int ymin, int ymax)
 {
- int indexx,i,j,left,len,ulx,uly,x,y,bm_h,bm_w;
+ int indexx,i,j,left,ulx,uly,x,y,bm_h,bm_w;
  unsigned int rectw, recth,w,h;
  FileRec *file;
  int recty,rectx;
@@ -317,7 +319,6 @@ static void drawPart(IconFileListWidget ilw,
 	/* Draw one element here */
 
 	file=FL.files[indexx];
-	len=strlen(file->name);
 
 	gc=FL.gc_norm;
 	if (IL.hil_entry==indexx) {
@@ -328,13 +329,12 @@ static void drawPart(IconFileListWidget ilw,
 	  gc=FL.gc_invert;
         }
 
-// TODO: shorten the strings if longer than max_level_width
 	XDrawString(di,win,
 		gc,
-		ulx+(bm_w-XTextWidth(FL.font,file->name,len))/2,
+		ulx+(bm_w-file->width)/2,
 		uly+IL.liney,
-		file->name,
-		len);
+		file->reducedname,
+		file->reducedlen);
 
 	w = file->mime_type->data->bm_width;
 	h = file->mime_type->data->bm_height;
