@@ -4,7 +4,7 @@
  *  The TextField Widget
  *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>
  
- *  modified 2004,2005,2006 by Bernhard R. Link (see Changelog)
+ *  modified 2004,2005,2006,2007 by Bernhard R. Link (see Changelog)
  *  ----------------------------------------------------------------------
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Library Public License as published by
@@ -62,6 +62,7 @@
 
 #include "global.h"
 #include "TextFieldP.h"
+#include "FmErrors.h"
 
 /* Macro definitions */
 
@@ -1083,8 +1084,6 @@ static Boolean GetSelectionFromArgs(Widget w,
 		                    String *args, Cardinal *nargs,
 				    Atom *selection)
 {
-char mess[128];
-
 if (*nargs>1) {
 	XtWarning("TextField: Too many arguments");
 	return(False);
@@ -1096,9 +1095,10 @@ if (*nargs>0) {
 	  case 's': case 'S': *selection=XA_SECONDARY;	  	     break;
 	  case 'c': case 'C': *selection=XA_CLIPBOARD(XtDisplay(w)); break;
 
-	  default:  
-		(void)sprintf(mess,"TextField: %s invalid selection name",args[0]);
-		XtWarning(mess);
+	  default:
+		XtAppWarningFmt(XtWidgetToApplicationContext(w),
+				"TextField: %s invalid selection name",
+				args[0]);
 		return(False);
 	}
 } else { /* no arguments */
@@ -1813,7 +1813,6 @@ static void Initialize(UNUSED(Widget treq), Widget tnew,
 {
  TextFieldWidget tfw=(TextFieldWidget)tnew;
  Dimension       minh;
- char		 	 mess[256];
  int		 	 len;
 
  /* get Xt's multiclick time */
@@ -1848,9 +1847,10 @@ static void Initialize(UNUSED(Widget treq), Widget tnew,
  	TF.buffer[0]=TF.buffer[TF.maxlen]=(char)0;
 
  	if (strlen(TF.string)>TF.maxlen) {
-		(void)sprintf(mess,"TextField: Buffer full (%i chars)",(int)TF.maxlen);
 		XBell(XtDisplay((Widget)tfw),100);
-		XtWarning(mess);
+		XtAppWarningFmt(XtWidgetToApplicationContext(tnew),
+				"TextField: Buffer full (%i chars)",
+				(int)TF.maxlen);
  	}
 
  	if (TF.string) (void)strncpy(TF.buffer,TF.string,TF.maxlen);
@@ -1897,8 +1897,9 @@ static void Initialize(UNUSED(Widget treq), Widget tnew,
 
  if (tfw->core.height < minh) {
    if (tfw->core.height > 0) {
-     (void)sprintf(mess,"TextField: Height too small (using %i)",(int)minh);
-     XtWarning(mess);
+     XtAppWarningFmt(XtWidgetToApplicationContext(tnew),
+		     "TextField: Height too small (using %i)",
+		     (int)minh);
    }
    tfw->core.height=minh;
  }
@@ -2406,7 +2407,6 @@ static Boolean CvtByFormat(Display *di, XrmValuePtr args, Cardinal *nargs,
 {
  String  fmt;
  Boolean rval,fromString;
- char    mess[256];
  int	 typ=0;
 
  int     iarg=0;
@@ -2419,32 +2419,36 @@ static Boolean CvtByFormat(Display *di, XrmValuePtr args, Cardinal *nargs,
 	     "String to CFormat conversion needs 1 extra arg",
 	     (String*)NULL,(Cardinal*)NULL);
  }
- (void)sprintf(mess,"CvtByFormat: ");
 
  fmt=*(String*)args[0].addr;
  fromString=(Boolean)(unsigned)(*(XtPointer*)args[1].addr);
  rval=True;
 
  if (fmt==NULL) {
-  (void)sprintf(mess+13,"%s","no format string");
-  XtWarning(mess); rval=False;
+  XtAppWarning(XtDisplayToApplicationContext(di),
+	       "CvtByFormat: no format string");
+  rval=False;
  } else if ((typ=strlen(fmt)-1)>80) {
-  (void)sprintf(mess+13,"%s","format too long");
-  XtWarning(mess); rval=False;
+  XtAppWarning(XtDisplayToApplicationContext(di),
+	       "CvtByFormat: format too long");
+  rval=False;
  } else if (fmt[0]!='%') {
-  (void)sprintf(mess+13,"%s","possibly no format (missing %%)");
-  XtWarning(mess); rval=False;
+  XtAppWarning(XtDisplayToApplicationContext(di),
+	       "CvtByFormat: possibly no format (missing %%)");
+  rval=False;
  }
- 
- 
+
+
  if (from->addr==NULL) {
-  (void)sprintf(mess+13,"%s","no source address");
-  XtWarning(mess); rval=False;
- } 
+  XtAppWarning(XtDisplayToApplicationContext(di),
+	       "CvtByFormat: no source address");
+   rval=False;
+ }
  if (to->addr==NULL) {
-  (void)sprintf(mess+13,"%s","no target address");
-  XtWarning(mess); rval=False;
- } 
+  XtAppWarning(XtDisplayToApplicationContext(di),
+	       "CvtByFormat: no target address");
+  rval=False;
+ }
  if (rval) {
   if (fromString) {
    rval=(sscanf(from->addr,fmt,to->addr)==1);
@@ -2473,7 +2477,7 @@ static Boolean CvtByFormat(Display *di, XrmValuePtr args, Cardinal *nargs,
 	       if (rval) {
                  if (fmt[typ-1]=='l') (void)sprintf(to->addr,fmt,larg);
                  else		      (void)sprintf(to->addr,fmt,iarg);
-		
+
 		 if (strlen(to->addr) > to->size) { /* !! String overflow !! */
   			XtErrorMsg("buffer overflow", "CvtByFormat",
 	     			"XtToolkitError",
@@ -2482,7 +2486,7 @@ static Boolean CvtByFormat(Display *di, XrmValuePtr args, Cardinal *nargs,
 
 		 }
 
-	       } 
+	       }
 	       break;
 
     case 'f':
@@ -2508,20 +2512,23 @@ static Boolean CvtByFormat(Display *di, XrmValuePtr args, Cardinal *nargs,
 	       }
 	       break;
 
-    default:   (void)sprintf(mess+13,"%s","wrong conversion character");
-  	       XtWarning(mess); rval=False;
+    default:   XtAppWarning(XtDisplayToApplicationContext(di),
+			    "CvtByFormat: wrong conversion character");
+	       rval=False;
 	       break;
   }
  }
 
  if (!rval) {
   if (fromString) {
+    char *mess = XtMalloc(20+strlen(XtRCFormat)+strlen(fmt));
     (void)sprintf(mess,"%s using format '%s'",XtRCFormat,fmt);
     XtDisplayStringConversionWarning(di,(char*)from->addr,mess);
+    XtFree(mess);
   } else {
-    (void)sprintf(mess,"Cannot convert type %s to string %s using format %s",
-	XtRCFormat,(char*)from->addr,fmt);
-    XtWarning(mess);
+    XtAppWarningFmt(XtDisplayToApplicationContext(di),
+	       "CvtByFormat: Cannot convert type %s to string %s using format %s",
+	       XtRCFormat, (char*)from->addr, fmt);
   }
  }
  return(rval);
