@@ -1,9 +1,12 @@
 /*
- *  Enhancements to the X-File Manager XFM-1.3.2 (The TextFileList Widget)
+ *  This file is part of X-File Manager XFM
  *  ----------------------------------------------------------------------
+ *  Enhancements to the X-File Manager XFM-1.3.2 (The TextFileList Widget)
  
  *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>
-
+ *
+ *  modified 2005,2006 by Bernhard R. Link (see Changelog for details)
+ *  ----------------------------------------------------------------------
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -16,9 +19,10 @@
 
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
+#include <xfmconfig.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -30,8 +34,9 @@
 
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
+#include "global.h"
 #include "TextFileListP.h"
-#include <X11/Xaw/XawInit.h>
+#include <X11/Xaw3d/XawInit.h>
 
 static XtResource tl_resources[] = {
 #define offset(field) XtOffsetOf(TextFileListRec, textFileList.field)
@@ -45,7 +50,7 @@ static XtResource tl_resources[] = {
     { XtNtabSep, XtCTabSep, XtRInt, sizeof(int),
 	offset(tabsep), XtRImmediate, (XtPointer)4},
     { XtNshowInode, XtCShowInode, XtRBoolean, sizeof(Boolean),
-	offset(ino_s), XtRImmediate, (XtPointer)True},
+	offset(ino_s), XtRImmediate, (XtPointer)False},
     { XtNshowType, XtCShowType, XtRBoolean, sizeof(Boolean),
 	offset(dev_s), XtRImmediate, (XtPointer)True},
     { XtNshowLinks, XtCShowLinks, XtRBoolean, sizeof(Boolean),
@@ -95,11 +100,48 @@ static XtActionsRec actions[] =
     {"unhighlight",	unhighlight},
 };
 
-static char translations[] ="";
-/*
-"<Key>:		textFileList()	\n\
-";
-*/
+static char translations[] =
+"<Enter>		: dispatchDirFile(fileHighlight,fileMaybeHighlight,\"\")\n"
+"<Leave>		: resetCursor()\n"
+"Shift  <Btn1Down>,<Btn1Up> : fileToggle()\n"
+"<Btn1Up>(2)		: dispatchDirFile(fileOpenDir,fileExecAction,\"\")\n"
+"<Btn1Down>,<Btn1Up>	: fileSelect()\n"
+"<Btn1Down>,<Motion>	: fileBeginDrag(1,move)\n"
+"<Btn2Down>,<Btn2Up>	: fileToggle()\n"
+"<Btn2Down>,<Motion>	: fileBeginDrag(2,copy)\n"
+"Any<Btn3Down>		: grabMouseForPopup(dispatchDirFile,dirPopup,filePopup,\"\")\n"
+"Shift   <Key>n		: notify(*folder*new)\n"
+"Shift   <Key>g		: notify(*folder*goto)\n"
+"Shift   <Key>h		: notify(*folder*home)\n"
+"Shift   <Key>u		: notify(*folder*up)\n"
+"Shift   <Key>c		: notify(*folder*clone)\n"
+"Shift   <Key>q		: notify(*folder*close)\n"
+"Ctrl    <Key>r		: notify(*view*tree)\n"
+"Ctrl    <Key>i		: notify(*view*icons)\n"
+"Ctrl    <Key>t		: notify(*view*text)\n"
+"Ctrl    <Key>n		: notify(\"*view*sort by name\")\n"
+"Ctrl    <Key>s		: notify(\"*view*sort by size\")\n"
+"Ctrl    <Key>d		: notify(\"*view*sort by mtime\")\n"
+"Ctrl    <Key>f		: notify(*view*filter)\n"
+"Ctrl    <Key>h		: notify(\"*view*hide folders\")\n"
+"Ctrl    <Key>m		: notify(\"*view*mix folders/files\")\n"
+"Ctrl    <Key>u		: notify(\"*view*show hidden files\")\n"
+#ifdef ENHANCE_LOG
+"Ctrl	 <Key>l		: notify(\"*view*show log\")\n"
+#endif
+"<Key>n			: notify(*file*new)\n"
+"<Key>m			: notify(*file*move)\n"
+"<Key>c			: notify(*file*copy)\n"
+"<Key>l			: notify(*file*link)\n"
+"<Key>d			: notify(*file*delete)\n"
+"<Key>Delete		: notify(*file*delete)\n"
+"<Key>BackSpace		: notify(*file*delete)\n"
+"<Key>s			: notify(*file*select)\n"
+"<Key>a			: notify(\"*file*select all\")\n"
+"<Key>u			: notify(\"*file*deselect all\")\n"
+"<Key>o			: notify(\"*file*own Selection\")\n"
+"<Key>x			: notify(*file*xterm)\n"
+"<Key>q			: notify(*file*quit)\n";
 
 TextFileListClassRec textFileListClassRec = {
   { /* core fields */
@@ -154,10 +196,8 @@ WidgetClass textFileListWidgetClass = (WidgetClass)&textFileListClassRec;
 #define TL (tlw->textFileList)
 #define FL (tlw->fileList)
 
-static void tlInitialize(treq,tnew,args,n_args)
- Widget	  treq,tnew;
- ArgList  args;
- Cardinal *n_args;
+static void tlInitialize(UNUSED(Widget treq), Widget tnew,
+                         UNUSED(ArgList args), UNUSED(Cardinal *n_args))
 {
  int  width,height,tmp;
  TextFileListWidget tlw=(TextFileListWidget)tnew;
@@ -191,7 +231,7 @@ static void tlInitialize(treq,tnew,args,n_args)
  if ((int)tlw->core.height<height) tlw->core.height=height;
 }
 
-static void tlExpose(Widget w,XEvent *ev,Region reg)
+static void tlExpose(Widget w, XEvent *ev, UNUSED(Region reg))
 {
  TextFileListWidget tlw=(TextFileListWidget)w;
  int imin,imax;
@@ -211,7 +251,8 @@ static void drawPart(TextFileListWidget tlw, int imin, int imax)
  Display *di=XtDisplay((Widget)tlw);
  Window  win=XtWindow((Widget)tlw);
  GC	 gc,gcnorm=FL.gc_norm;
- char    buf[BUFSIZ], *chpt;
+ char    buf[BUFSIZ];
+ const char *chpt;
  struct  passwd *pw;
  struct  group  *gr;
 
@@ -298,7 +339,14 @@ static void drawPart(TextFileListWidget tlw, int imin, int imax)
 	   XDrawString(di,win,gcnorm,ino_x+fill,liney,buf,len);
 	}
 	if (TL.dev_s) {
-	   if      (file->sym_link) 		   chpt="l";
+	   if(file->info.symlink) {
+		   if (S_ISDIR(file->stats.st_mode))  	   chpt="ld";
+		   else if (S_ISCHR(file->stats.st_mode))  chpt="lc";
+		   else if (S_ISBLK(file->stats.st_mode))  chpt="lb";
+		   else if (S_ISFIFO(file->stats.st_mode)) chpt="lp";
+		   else if (S_ISSOCK(file->stats.st_mode)) chpt="ls";
+		   else 				   chpt="l";
+	   }
 	   else if (S_ISDIR(file->stats.st_mode))  chpt="d";
 	   else if (S_ISCHR(file->stats.st_mode))  chpt="c";
 	   else if (S_ISBLK(file->stats.st_mode))  chpt="b";
@@ -306,8 +354,8 @@ static void drawPart(TextFileListWidget tlw, int imin, int imax)
 	   else if (S_ISSOCK(file->stats.st_mode)) chpt="s";
 	   else 				   chpt="-";
 
-	   fill=TL.dev_w-XTextWidth(FL.font,chpt,1);
-	   XDrawString(di,win,gcnorm,dev_x+fill,liney,chpt,1);
+	   fill=TL.dev_w-XTextWidth(FL.font,chpt,len=strlen(chpt));
+	   XDrawString(di,win,gcnorm,dev_x+fill,liney,chpt,len);
 	}
 	if (TL.umode_s) {
 	   makePermissionsString(buf,file->stats.st_mode);
@@ -360,8 +408,8 @@ static void drawPart(TextFileListWidget tlw, int imin, int imax)
 #define TN (new->textFileList)
 #define TC (current->textFileList)
 
-static Boolean tlSetValues(Widget c, Widget r, Widget super,
-	ArgList args, Cardinal *nargs)
+static Boolean tlSetValues(Widget c, UNUSED(Widget r), Widget super,
+	UNUSED(ArgList args), UNUSED(Cardinal *nargs))
 {
 int rval=False;
 TextFileListWidget current=(TextFileListWidget)c;
@@ -436,7 +484,6 @@ static XtGeometryResult tlQueryGeometry(Widget w,
 	return XtGeometryAlmost;
 }
 
-/*ARGSUSED*/
 static void highlight(Widget w, XEvent* ev, String* args, Cardinal* nargs)
 {
 TextFileListWidget tlw=(TextFileListWidget)w;
@@ -455,8 +502,8 @@ TL.hil_entry=i;
 if (i>=0)    drawPart(tlw,i,i);
 }
 
-/*ARGSUSED*/
-static void unhighlight(Widget w, XEvent* ev, String* args, Cardinal* nargs)
+static void unhighlight(Widget w, UNUSED(XEvent* ev),
+                        UNUSED(String* args), UNUSED(Cardinal* nargs))
 {
 TextFileListWidget tlw=(TextFileListWidget)w;
 int i=TL.hil_entry;
@@ -480,7 +527,7 @@ drawPart(tlw,i,i);
 
 /* private procedures */
 
-static int tlFindEntry(FileListWidget flw, int x, int y, XEvent *ev)
+static int tlFindEntry(FileListWidget flw, int x, int y, UNUSED(XEvent *ev))
 {
  TextFileListWidget tlw=(TextFileListWidget)flw;
  int i;
